@@ -166,6 +166,9 @@ class FunkLoadTestCase(unittest.TestCase):
     #
     def _connect(self, url, params, ok_codes, rtype, description):
         """Handle fetching, logging, errors and history."""
+        if params is None and rtype == 'post':
+            # enable empty post
+            params = []
         t_start = time.time()
         try:
             response = self._browser.fetch(url, params, ok_codes=ok_codes)
@@ -245,9 +248,10 @@ class FunkLoadTestCase(unittest.TestCase):
 
         if method == 'get' and params:
             url = url_in + '?' + urlencode(params)
-            params = None
         else:
             url = url_in
+        if method == 'get':
+            params = None
 
         if method == 'get':
             self.logd('GET: %s\n\tPage %i: %s ...' % (url, self.steps,
@@ -268,6 +272,8 @@ class FunkLoadTestCase(unittest.TestCase):
                 # Figure the location - which may be relative
                 newurl = response.headers['Location']
                 url = urljoin(url_in, newurl)
+                # save the current url as the base for future redirects
+                url_in = url
                 self.logd(' Load redirect link: %s' % url)
                 response = self._connect(url, None, ok_codes, 'redirect', None)
                 max_redirect_count -= 1
@@ -497,20 +503,29 @@ class FunkLoadTestCase(unittest.TestCase):
             return response.body
         return ''
 
-    def listHref(self, pattern=None):
+    def listHref(self, url_pattern=None, content_pattern=None):
         """Return a list of href anchor url present in the last html response.
 
-        Filtering href using the pattern regex if present."""
+        Filtering href with url pattern or link text pattern."""
         response = self._response
         ret = []
         if response is not None:
             a_links = response.getDOM().getByName('a')
             if a_links:
-                ret = [getattr(x, 'href', '') for x in a_links]
-            if pattern is not None:
-                pat = re.compile(pattern)
-                ret = [href for href in ret if pat.search(href) is not None]
-        return ret
+                for link in a_links:
+                    try:
+                        ret.append((link.getContentString(), link.href))
+                    except AttributeError:
+                        pass
+            if url_pattern is not None:
+                pat = re.compile(url_pattern)
+                ret = [link for link in ret
+                       if pat.search(link[1]) is not None]
+            if content_pattern is not None:
+                pat = re.compile(content_pattern)
+                ret = [link for link in ret
+                       if link[0] and (pat.search(link[0]) is not None)]
+        return [link[1] for link in ret]
 
     def getLastBaseUrl(self):
         """Return the base href url."""
@@ -789,7 +804,10 @@ class FunkLoadTestCase(unittest.TestCase):
             except KeyboardInterrupt:
                 raise
             except:
-                result.addError(self, self._TestCase__exc_info())
+                if sys.version_info >= (2, 5):
+                    result.addError(self, self._exc_info())
+                else:
+                    result.addError(self, self._TestCase__exc_info())
                 self.test_status = 'Error'
                 self._log_result(t_start, time.time())
                 return
